# -*- coding: utf-8 -*-
"""
Accepts a DSM model in the form of a .pkl file and assess it using a variety of measures from the literature and report the performance.

@author: Max
"""
# All datasets are stored in .csv format
import csv
import sys, getopt
from composes.utils import io_utils
from composes.similarity.cos import CosSimilarity
from scipy.stats import spearmanr
from scipy.stats import pearsonr


def rubensteingoodenough(model):
    """Accepts a DISSECT model and compares its performance against the rubensteingoodenough dataset.
    It returns the result of this comparison -- a SpearmanR correlation coefficient."""

    # The format of the file is <noun1>,<noun2>,<float-similarity>
    # The nouns have double quotes around them
    words1 = []
    words2 = []
    similarities = []
    try:
        with open('../datasets/rubensteingoodenough.csv', 'rb') as csvfile:
            rreader = csv.reader(csvfile, delimiter=',', quotechar='|')
            for row in rreader:
                words1.append(row[0])
                words2.append(row[1])
                similarities.append(float(row[2]))
    except IOError:
        print 'The file \'rubensteingoodenough.csv\' was not found in the local directory'

    # To remove the double quotes and append 'NN', which is needed to query the model
    # words1 = ['{0}_NN'.format(rWord.replace("\"","")) for rWord in words1]
    #words2 = ['{0}_NN'.format(rWord.replace("\"","")) for rWord in words2]

    # make a vector of the cosine similarities between the words in words1 and words2
    msims = []
    for i in range(len(words1)):
        msims.append(model.get_sim(words1[i], words2[i], CosSimilarity()))
    # Ultimately, the quality of the model's predictions is the correlation between the similarity scores and
    # the cosine similarities
    rSpearman = spearmanr(msims, similarities)
    # For whatever reason, the DM data uses Pearson
    rPearson = pearsonr(msims, similarities)
    return rSpearman


def men(model):
    """Accepts a DISSECT model and compares its performance against the MEN dataset.
    It returns the result of this comparison -- a SpearmanR correlation coefficient."""

    # The format of the file is <"word1-[n/j/v]">,<"word2--[n/j/v]">,<int>
    # The nouns have double quotes around them
    words1 = []
    words2 = []
    similarities = []
    try:
        with open('../datasets/men.csv', 'rb') as csvfile:
            rreader = csv.reader(csvfile, delimiter=',', quotechar='|')
            for row in rreader:
                words1.append(row[0])
                words2.append(row[1])
                similarities.append(int(row[2]))
    except IOError:
        print 'The file \'men.txt\' was not found in the local directory'

    # To remove the double quotes and append NN/JJ/VB as appropriate
    # words1 = ['{0}'.format(mWord.replace("\"","").replace("-n","_NN").replace("-v","_VB").replace("-j","_JJ")) for mWord in words1]
    #words2 = ['{0}'.format(mWord.replace("\"","").replace("-n","_NN").replace("-v","_VB").replace("-j","_JJ")) for mWord in words2]

    # To address 'clothes' and 'sunglasses'
    #words1 = [word.replace("clothes_NN","clothes_NNS").replace("sunglasses_NN","sunglasses_NNS") for word in words1]
    #words2 = [word.replace("clothes_NN","clothes_NNS").replace("sunglasses_NN","sunglasses_NNS") for word in words2]

    # make a vector of the cosine similarities between the words in words1 and words2
    msims = []
    for i in range(len(words1)):
        msims.append(model.get_sim(words1[i], words2[i], CosSimilarity()))
    # Ultimately, the quality of the model's predictions is the correlation between the similarity scores and
    # the cosine similarities
    rSpearman = spearmanr(msims, similarities)
    # For whatever reason, the DM data uses Pearson
    rPearson = pearsonr(msims, similarities)
    return rSpearman


def wordsim353(model):
    """Accepts a DISSECT model and compares its performance against the wordsim353 dataset.
    It returns the result of this comparison -- a SpearmanR correlation coefficient."""

    # The format of the file is <noun1>,<noun2>,<float-similarity>
    # The nouns have double quotes around them
    words1 = []
    words2 = []
    similarities = []
    try:
        with open('../datasets/wordsim353.csv', 'rb') as csvfile:
            rreader = csv.reader(csvfile, delimiter=',', quotechar='|')
            for row in rreader:
                words1.append(row[0])
                words2.append(row[1])
                similarities.append(float(row[2]))
    except IOError:
        print 'The file \'wordsim353.csv\' was not found in the local directory'

    # To remove the double quotes and append 'NN', which is needed to query the model
    # words1 = ['{0}_NN'.format(word.replace("\"","")) for word in words1]
    #words2 = ['{0}_NN'.format(word.replace("\"","")) for word in words2]

    # To address 'clothes' and 'sunglasses'
    #words1 = [word.replace("clothes_NN","clothes_NNS").replace("sunglasses_NN","sunglasses_NNS") for word in words1]
    #words2 = [word.replace("clothes_NN","clothes_NNS").replace("sunglasses_NN","sunglasses_NNS") for word in words2]

    # make a vector of the cosine similarities between the words inwords1andwords2
    msims = []
    for i in range(len(words1)):
        msims.append(model.get_sim(words1[i], words2[i], CosSimilarity()))
    # Ultimately, the quality of the model's predictions is the correlation between the similarity scores and
    # the cosine similarities
    rSpearman = spearmanr(msims, similarities)
    # For whatever reason, the DM data uses Pearson
    rPearson = pearsonr(msims, similarities)
    return rSpearman


def main(argv):
    modelfile = ''

    try:
        opts, args = getopt.getopt(argv, "hm:", ["mod="])
    except getopt.GetoptError:
        print 'weighting-test.py -m <modelfile>'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'weighting-test.py -m <modelfile>'
            sys.exit()
        elif opt in ("-m", "--mod"):
            modelfile = arg

            # load the input file
    # model = io_utils.load("./" + modelfile)
    model = io_utils.load("/home/lotsteim/data/biarcs/ppmi/outfulldim.pkl")

    # Testing Semantic Relatedness
    # Each test in this series looks at the correlation between human raters similarity scores with the
    # Cosine similarity of two word vectors.
    semanticPerformance = []

    # The first of these tests uses the Rubenstein-Goodenough dataset.
    semanticPerformance.append(rubensteingoodenough(model))
    print("Done with Rubenstein")
    # The second uses the wordsim353 dataset
    semanticPerformance.append(wordsim353(model))
    print("Done with wordsim")
    # The final one uses the men dataset
    semanticPerformance.append(men(model))

    print semanticPerformance

# Testing
if __name__ == "__main__":
    main(sys.argv[1:])
