"""
Transforms co-occurrence matrix into a Space object, replacing frequency with a weighted value derived from either
PPMI, PLMI, EPMI or PLOG.

Created on Wed May 21 13:23:07 2014

@author: Max
"""

# ex03.py
#-------
import os
from composes.semantic_space.space import Space
from composes.transformation.scaling.ppmi_weighting import PpmiWeighting
from composes.transformation.scaling.epmi_weighting import EpmiWeighting
from composes.transformation.scaling.plmi_weighting import PlmiWeighting
from composes.transformation.scaling.plog_weighting import PlogWeighting
from composes.transformation.dim_reduction.svd import Svd

from composes.utils import io_utils
import sys
import argparse
import time


def main(argv):
    parser = argparse.ArgumentParser(description='Transforms co-occurrence matrix into a Space object,' +
                                                 ' replacing freq. with weighted freq.')
    parser.add_argument('-m', action='store', dest='matrixfile',
                        help='The name of the matrix file', required=True)
    parser.add_argument('-r', action='store', dest='rowfile',
                        help='The name of the rows file', required=True)
    parser.add_argument('-c', action='store', dest='colfile',
                        help='The name of the cols file', required=True)
    parser.add_argument('-w', action='store', dest='weightmethod',
                        help='0=All Weight Methods, 1=PPMI, 2= PLMI, 3=EPMI, 4=PLOG', default=0, type=int)

    args = vars(parser.parse_args())

    # build unweighted space
    my_space = Space.build(data=args['matrixfile'],
                           rows=args['rowfile'],
                           cols=args['colfile'],
                           format="sm")

    if (args['weightmethod'] == 0):
        startTime = round(time.time())
        # build PPMI weighted space and write to disk
        ppmiSpace = my_space.apply(PpmiWeighting())
        io_utils.save(ppmiSpace, "./ppmi.pkl")
        print("Done with PPMI!")
        print(int(round(time.time()) - startTime), "s")
        startTime = round(time.time())

        # build PLMI weighted space and write to disk
        plmiSpace = my_space.apply(PlmiWeighting())
        io_utils.save(plmiSpace, "./plmi.pkl")
        print("Done with PLMI!")
        print(int(round(time.time()) - startTime), "s")
        startTime = round(time.time())

        # build EPMI weighted space and write to disk
        epmiSpace = my_space.apply(EpmiWeighting())
        io_utils.save(epmiSpace, "./epmi.pkl")
        print("Done with EPMI!")
        print(int(round(time.time()) - startTime), "s")
        startTime = round(time.time())

        # build PLOG weighted space and write to disk
        plogSpace = my_space.apply(PlogWeighting())
        io_utils.save(plogSpace, "./plog.pkl")
        print("Done with PLOG!")
        print(int(round(time.time()) - startTime), "s")
    else:
        if (args['weightmethod'] == 1):
            newspace = my_space.apply(PpmiWeighting())
            filename = "./ppmi.pkl"
        elif (args['weightmethod'] == 2):
            newspace = my_space.apply(PlmiWeighting())
            filename = "./plmi.pkl"
        elif (args['weightmethod'] == 3):
            newspace = my_space.apply(EpmiWeighting())
            filename = "./epmi.pkl"
        elif (args['weightmethod'] == 4):
            newspace = my_space.apply(PlogWeighting())
            filename = "./plog.pkl"
        io_utils.save(newspace, filename)

if __name__ == "__main__":
    main(sys.argv[1:])
